@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include <algorithm>
@@ -113,7 +113,7 @@ void Transformation::show_error( const char * program_name, const char * arg ) c
   std::fputs( "Valid transformation names are:", stderr );
   for( int i = 0; T_table[i].name != 0; ++i )
     std::fprintf( stderr, "  %s", T_table[i].name );
-  std::fputs( "\n", stderr );
+  std::fputs( "\nRotations are made counter-clockwise.\n", stderr );
   }
 
 
