@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include <algorithm>
@@ -22,14 +22,14 @@
 #include "common.h"
 #include "rectangle.h"
 #include "ucs.h"
+#include "bitmap.h"
 #include "block.h"
-#include "blockmap.h"
 #include "profile.h"
 #include "feats.h"
 
 
 Features::Features( const Block & b ) throw()
-  : _block( &b ), _blockmap( b.blockmap() ), _hbars( -1 ), _vbars( -1 ),
+  : _block( &b ), _hbars( -1 ), _vbars( -1 ),
     hscan_valid( false ), vscan_valid( false ), lp( b, Profile::left ),
     tp( b, Profile::top ), rp( b, Profile::right ), bp( b, Profile::bottom ),
     hp( b, Profile::height ), wp( b, Profile::width )
@@ -41,7 +41,6 @@ int Features::hbars() const throw()
   if( _hbars < 0 )
     {
     const Block & b = *_block;
-    const Blockmap & bm = *_blockmap;
     int state = 0, begin = 0, l = 0, r = 0, limit = wp.max() / 2;
     std::vector< int > count( b.height(), 0 );
     _hbars = 0;
@@ -52,7 +51,7 @@ int Features::hbars() const throw()
       int & maxcount = count[row-b.top()];
       for( col = b.left(); col <= b.right(); ++col )
         {
-        if( bm.id( row, col ) == b.id() )
+        if( b.get_bit( row, col ) )
           { ++c; x = col; if( col < b.right() ) continue; }
         if( c > maxcount ) { maxcount = c; rt = x; lt = rt - c + 1; }
         c = 0;
@@ -90,7 +89,6 @@ int Features::vbars() const throw()	// FIXME small gaps not detected
   if( _vbars < 0 )
     {
     const Block & b = *_block;
-    const Blockmap & bm = *_blockmap;
     int state = 0, begin = 0, limit = b.height();
     limit -= ( b.height() < 40 ) ? 3 : b.height() / 10;
     _vbars = 0;
@@ -100,10 +98,10 @@ int Features::vbars() const throw()	// FIXME small gaps not detected
       int c = 0, c2 = 0, count = 0;
       for( int row = b.top() + 1; row < b.bottom(); ++row )
         {
-        if( bm.id( row, col ) == b.id() )
+        if( b.get_bit( row, col ) )
           { ++c; if( row < b.bottom() - 1 ) continue; }
-        else if( ( col > b.left() && bm.id( row, col - 1 ) == b.id() ) ||
-                 ( col < b.right() && bm.id( row, col + 1 ) == b.id() ) )
+        else if( ( col > b.left() && b.get_bit( row, col - 1 ) ) ||
+                 ( col < b.right() && b.get_bit( row, col + 1 ) ) )
           { ++c; ++c2; if( row < b.bottom() - 1 ) continue; }
         if( c > count ) { count = c; } c = 0;
         }
@@ -137,12 +135,12 @@ int Features::vbars() const throw()	// FIXME small gaps not detected
 
 
 // return the number of horizontal traces crossing every column
+//
 const std::vector< int > & Features::hscan() const throw()
   {
   if( !hscan_valid )
     {
     const Block & b = *_block;
-    const Blockmap & bm = *_blockmap;
     int state = 0;
     hscan_valid = true; _hscan.resize( b.width() );
 
@@ -151,7 +149,7 @@ const std::vector< int > & Features::hscan() const throw()
       _hscan[col-b.left()] = 0;
       for( int row = b.top(); row <= b.bottom(); ++row )
         {
-        bool black = ( bm.id( row, col ) == b.id() );
+        bool black = b.get_bit( row, col );
         switch( state )
           {
           case 0: if( black ) state = 1; break;
@@ -166,12 +164,12 @@ const std::vector< int > & Features::hscan() const throw()
 
 
 // return the number of vertical traces crossing every row
+//
 const std::vector< int > & Features::vscan() const throw()
   {
   if( !vscan_valid )
     {
     const Block & b = *_block;
-    const Blockmap & bm = *_blockmap;
     int state = 0;
     vscan_valid = true; _vscan.resize( b.height() );
 
@@ -180,7 +178,7 @@ const std::vector< int > & Features::vscan() const throw()
       _vscan[row-b.top()] = 0;
       for( int col = b.left(); col <= b.right(); ++col )
         {
-        bool black = ( bm.id( row, col ) == b.id() );
+        bool black = b.get_bit( row, col );
         switch( state )
           {
           case 0: if( black ) state = 1; break;
@@ -209,6 +207,10 @@ int Features::test_misc() const throw()
       b.seek_left( b.vcenter(), b.hcenter() ) <= b.left() )
     return '7';
 
+  if( b.height() > 2 * b.width() && rp.increasing( 1 ) && tp.decreasing( 1 ) &&
+      lp.iscpit( 25 ) )
+    return '1';
+
   if( tp.minima( b.height() / 4 ) == 1 && bp.minima( b.height() / 4 ) == 1 )
     {
     if( b.height() > 2 * wp.max() )
