@@ -13,13 +13,12 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 class Features
   {
   const Block * _block;		// Block to witch this features belong
-  const Blockmap * _blockmap;	// Blockmap to which the block belongs
   mutable int _hbars, _vbars;
   mutable bool hscan_valid, vscan_valid;
   mutable std::vector< Rectangle > _hbar, _vbar;
@@ -31,7 +30,6 @@ public:
   Features( const Block & b ) throw();
 
   const Block * block() const throw() { return _block; }
-  const Blockmap * blockmap() const throw() { return _blockmap; }
 
   const Rectangle & hbar( int i ) const throw() { return _hbar[i]; }
   const Rectangle & vbar( int i ) const throw() { return _vbar[i]; }
